import java.io.*;
import java.util.*;


/**
 *  FormCDataParser One Class  
 */

public class FormCDataParser {

    /**
    * Components
    */

    private Vector vData;
    private Vector term             = new Vector();
    private Vector course           = new Vector();
    private Vector section          = new Vector();
    private Vector instructor       = new Vector();
    private Vector courseKey        = new Vector();
    private Vector courseInstructor = new Vector();
    private Vector courseStudents   = new Vector();

    private static final int COURSE_NO   = 1;
    private static final int SECTION     = 2;
    private static final int TERM        = 4;
    private static final int COURSE_NAME = 7;
    private static final int INSTR		 = 8;
    private static final int INSTR_NAME  = 9;
    private static final int FIRST_NAME  = 15;
    private static final int MIDDLE_NAME = 16;
    private static final int LAST_NAME   = 17;

    int column[] = {TERM, COURSE_NO, COURSE_NAME, SECTION,
                    INSTR, INSTR_NAME,
                    FIRST_NAME, MIDDLE_NAME, LAST_NAME};


    /**
     * FormCDataParser() Constructor 
     */

    public FormCDataParser(BufferedReader br) {
        CsvLineConsumer cv = new CsvLineConsumer(br, column);
        vData = cv.getVData();
        processData();

    }

    /**
     * getTerm	  		Method
     *
     * @return 	  		Vector
     */

    public String[] getTerm() {
        Object [] objTerm = term.toArray();
        Arrays.sort(objTerm);

        String [] strTerm = new String[objTerm.length];

        for (int i = 0; i < strTerm.length; i++) {
            strTerm[i] = (String)objTerm[i];
        }

        return strTerm;
    }

    /**
     * getCourse	  		Method
     *
     * @return 	  		Vector
     */

    public String[] getCourse() {
        Object [] objCourse = course.toArray();
        Arrays.sort(objCourse);

        String [] strCourse = new String[objCourse.length];

        for (int i = 0; i < strCourse.length; i++) {
            strCourse[i] = (String)objCourse[i];
        }

        return strCourse;
    }

    /**
     * getSection	  		Method
     *
     * @return 	  		Vector
     */

    public String[] getSection() {
        Object [] objSection = section.toArray();
        Arrays.sort(objSection);

        String [] strSection = new String[objSection.length];

        for (int i = 0; i < strSection.length; i++) {
            strSection[i] = (String)objSection[i];
        }

        return strSection;
    }

    /**
     * getInstructor 		Method
     *
     * @return 	  		Vector
     */

    public String[] getInstructor() {
        Object [] objInstructor = instructor.toArray();
        Arrays.sort(objInstructor);

        String [] strInstructor = new String[objInstructor.length];

        for (int i = 0; i < strInstructor.length; i++) {
            strInstructor[i] = (String)objInstructor[i];
        }

        return strInstructor;
    }


    /**
     * getCourseKey  		Method
     *
     * @return 	  		Vector
     */

    public Vector getCourseKey() {

        return courseKey;
    }

    /**
     * getCourseInstructor  Method
     *
     * @return 	 	   	  Vector
     */

    public Vector getCourseInstructor() {

        return courseInstructor;
    }

    /**
     * getCourseStudents  Method
     *
     * @return 	 		Vector
     */

    public Vector getCourseStudents() {

        return courseStudents;
    }



    /**
      * processData Mehtod 
      *
      * @return 	 void
      */

    public void processData() {

        int count =  vData.size();

        for (int i = 0; i < count; i++) {
            String data[] = (String[])vData.elementAt(i);
            splitColumns(data);
        }

    }


    /**
      * splitColumns Mehtod 
      *
      * @parm	 	  String[]		record
      * @return      void
      */

    public void splitColumns(String data[]) {
        String termBuffer            = new String();
        StringBuffer courseBuffer    = new StringBuffer();
        StringBuffer sectionBuffer   = new StringBuffer();
        StringBuffer instBuffer      = new StringBuffer();
        StringBuffer courseKeyBuffer = new StringBuffer();
        StringBuffer studentBuffer   = new StringBuffer();

        int end = 0;

        for(int i = 0; i < data.length; i++) {

            switch (i) {
            case 0:
                termBuffer = data[i];
                addElement(term, data[i]);
                break;

            case 1:
                courseBuffer.append(data[i]);
                break;

            case 2:
                courseBuffer.append(" ");
                courseBuffer.append(data[i]);
                addElement(course, courseBuffer.toString());
                break;

            case 3:
                sectionBuffer.append(data[i]);
                addElement(section, sectionBuffer.toString());

                courseKeyBuffer.append(termBuffer);
                courseKeyBuffer.append(" ");
                courseKeyBuffer.append(courseBuffer.toString());
                courseKeyBuffer.append(" ");
                courseKeyBuffer.append(sectionBuffer.toString());
                addElement(courseKey,  courseInstructor,
                           courseStudents, courseKeyBuffer.toString());
                break;

            case 4:
                instBuffer.append(data[i]);
                break;

            case 5:
                instBuffer.append(" ");
                instBuffer.append(data[i]);
                addElement(instructor, instBuffer.toString());

                addElement(courseKey, courseInstructor,
                           courseKeyBuffer.toString(),
                           instBuffer.toString());
                break;

            case 6:
                studentBuffer.append(data[i]);
                break;

            case 7:
                studentBuffer.append(" ");
                studentBuffer.append(data[i]);
                break;

            case 8:
                studentBuffer.append(" ");
                studentBuffer.append(data[i]);
                addElement(courseKey, courseStudents,
                           courseKeyBuffer.toString(),
                           studentBuffer.toString());
                break;

            }
        }
    }


    /**
      * addElement	  Mehtod 
      *
      * @parm	 	  Vector
      * @parm	      data
      */

    private void addElement(Vector v, String data) {
        boolean dataExists = false;

        for (Enumeration e = v.elements(); e.hasMoreElements();) {
            if (data.equals((String)e.nextElement())) {
                dataExists = true;
                break;
            }
        }


        if(!dataExists) {
            v.addElement(data);
        }
    }


    /**
      * addElement	  Mehtod 
      *
      * @parm	 	  Vector
      * @parm	 	  Vector
      * @parm	      data
      */

    private void addElement(Vector key, Vector d1, Vector d2,  String data) {
        boolean dataExists = false;

        for (Enumeration ke = key.elements(); ke.hasMoreElements();) {
            if (data.equals((String)ke.nextElement())) {
                dataExists = true;
                break;
            }
        }


        if(!dataExists) {
            key.addElement(data);
            d1.addElement(new Vector());
            d2.addElement(new Vector());
        }
    }


    /**
      * addElement	  Mehtod 
      *
      * @parm	 	  Vector  		key
      * @parm	 	  Vector        data
      * @parm	      data          key 
      * @parm	      data			data
      * @return      void
      */

    private void addElement(Vector vkey, Vector v,
                            String skey, String data) {

        Enumeration ke = vkey.elements();

        for (int i = 0; ke.hasMoreElements(); i++) {
            if (skey.equals((String)ke.nextElement())) {
                Vector sv  = (Vector)v.elementAt(i);
                addElement(sv, data);
                break;
            }
        }

    }

}
