/*
 * $Id: OrderDAOSybase.java,v 1.9 2000/10/09 03:43:58 inder Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.order.ejb;

import java.util.Iterator;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.sun.j2ee.blueprints.customer.order.ejb.OrderDAOException;
import com.sun.j2ee.blueprints.customer.order.model.LineItem;
import com.sun.j2ee.blueprints.customer.util.Debug;
import com.sun.j2ee.blueprints.customer.util.DatabaseNames;

/**
 * This class represents the implementation of the
 * abstract methods of the base class OrderDAO for
 *  Sybase.
 */
public class OrderDAOSybase extends OrderDAO {

    public OrderDAOSybase() throws OrderDAOException {
        super();
    }

    /**
    * part of the OrderDAO abstract class
    */
    public void create() throws OrderDAOException {
        insertOrder();
        insertLineItem(orderId);
        insertOrderStatus(orderId);
    }

    /**
    * part of the OrderDAO abstract class
    */
    public void store() throws OrderDAOException {
        updateOrder();
        updateLineItem();
        updateOrderStatus();
    }

    private void insertOrder() throws OrderDAOException {
        Statement stmt = null;

        if (!isValidData())
            throw new OrderDAOException ("Illegal data values for insert");

        try {
           getDBConnection();
           stmt = dbConnection.createStatement();
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while getting connection and creating  statement");
        }

        String queryStr = "INSERT INTO " + DatabaseNames.ORDER_TABLE +
            "(userid,orderdate,"
            + "shipaddr1,shipaddr2,shipcity,shipstate,shipzip,shipcountry,"
            + "billaddr1,billaddr2,billcity,billstate,billzip,billcountry,"
            + "courier,totalprice,"
            + "shiptofirstname,shiptolastname,"
            + "billtofirstname,billtolastname,"
            + "creditcard,exprdate,cardtype)" + "VALUES ("
            + "'" + userId.trim() + "',"
            + "'"+ orderDate.getFullDateString().trim()+ "',"
            + "'"+shipToAddr.getStreetName1().trim() +"',";

        if (shipToAddr.getStreetName2() != null)
            queryStr += "'"+shipToAddr.getStreetName2().trim() +"',";
        else
            queryStr += "' ',";

        queryStr +=  "'"+shipToAddr.getCity().trim() + "',"
            + "'"+shipToAddr.getState().trim() + "',"
            + "'"+shipToAddr.getZipCode().trim() + "',"
            + "'"+shipToAddr.getCountry().trim() + "',"
            + "'"+billToAddr.getStreetName1().trim() +"',";

        if (billToAddr.getStreetName2() != null)
            queryStr += "'"+shipToAddr.getStreetName2().trim() +"',";
        else
            queryStr += "' ',";

        queryStr += "'"+billToAddr.getCity().trim() + "',"
            + "'"+billToAddr.getState().trim() + "',"
            + "'"+billToAddr.getZipCode().trim() + "',"
            + "'"+billToAddr.getCountry().trim() + "',"
            + "'"+carrier.trim() + "',"
            + "CONVERT(DECIMAL(10,2)," +totalPrice + "),"
            + "'"+shipToFirstName.trim() + "',"
            + "'"+shipToLastName.trim() + "',"
            + "'"+billToFirstName.trim() + "',"
            + "'"+billToLastName.trim() + "',"
            + "'"+chargeCard.getCardNo().trim() + "',"
            + "'"+chargeCard.getExpiryDateString().trim() + "',"
            + "'"+chargeCard.getCardType().trim() + "' )";

        Debug.println("queryString is: "+ queryStr);
        int resultCount = 0;
        try {
           resultCount = stmt.executeUpdate(queryStr);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while executing update for statement");
        }

        try{
           if ( resultCount != 1 )
               throw new OrderDAOException(
                "ERROR in ORDER_TABLE INSERT !! resultCount = " + resultCount);
           else {
               queryStr = "SELECT @@identity";
               ResultSet rs = stmt.executeQuery(queryStr);
               if ( !rs.next() ) {
                  throw new OrderDAOException("ERROR in selecting OrderId !!");
               } else {
                   orderId = rs.getInt(1);
                   if (orderId < 1)
                       throw new OrderDAOException
                           ("ERROR in getting OrderId !! orderId = "+ orderId);
               }
            }
          } catch(SQLException se) {
               throw new OrderDAOException("SQLException while inserting " +
                    "order " + orderId + " : \n" + se);
          } finally {
             try{
                 if(stmt != null) stmt.close();
                 closeConnection();
             } catch(SQLException se) {
                 throw new OrderDAOException("SQLException while closing statement and connection");
             }
          }
    }

    private void insertLineItem(int orderId) throws OrderDAOException {
        LineItem LI;
        int resultCount;

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            for (Iterator it = lineItems.iterator() ; it.hasNext() ;) {
                LI = (LineItem) it.next();
                if (LI == null)
                    throw new OrderDAOException("LineItem is null");
                String queryStr = "INSERT INTO "
                    + DatabaseNames.LINE_ITEM_TABLE
                    + "(orderid,linenum,itemid,quantity,unitprice) VALUES ("
                    + orderId + ","
                    + LI.getLineNo() + ","
                    + "'"+ LI.getItemNo().trim() + "',"
                    + LI.getQty() + ","
                    + LI.getUnitPrice() + ")";

                Debug.println("queryString is: "+ queryStr);
                resultCount = stmt.executeUpdate(queryStr);
                if ( resultCount != 1 )
                    throw new OrderDAOException
                        ("ERROR in LINE_ITEM_TABLE INSERT !! resultCount = "
                            + resultCount);
            }
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while inserting " +
                    "line item for order " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void insertOrderStatus(int orderId) throws OrderDAOException {

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr = "INSERT INTO "
                + DatabaseNames.ORDER_STATUS_TABLE
                + "(orderid,timestamp,status) VALUES ("
                + orderId + ","
                + "getdate(),"
                + "'" + status.trim() + "')";
            Debug.println("queryString is: "+ queryStr);
            int resultCount = stmt.executeUpdate(queryStr);
            if ( resultCount != 1 )
                throw new OrderDAOException
                    ("ERROR in ORDER_STATUS_TABLE INSERT !! resultCount = "
                        + resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while inserting " +
                    "order status for order " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private String formatPrice(double price){
        NumberFormat nf = NumberFormat.getInstance();
        DecimalFormat df = (DecimalFormat)nf;
        df.setMinimumFractionDigits(2);
        df.setMaximumFractionDigits(2);
        String pattern = "##.00";
        df.applyPattern(pattern);
        df.setDecimalSeparatorAlwaysShown(true);
        return df.format(price);
    }

    private void updateOrder() throws OrderDAOException {

        String queryStr = "UPDATE " + DatabaseNames.ORDER_TABLE
            + " SET userid  = " + "'" + userId.trim() + "',"
            + "orderdate = " + "'" + orderDate.getFullDateString().trim() + "',"
            + "shipaddr1 = " + "'" + shipToAddr.getStreetName1().trim() + "',";

        if (shipToAddr.getStreetName2() != null)
            queryStr += "shipaddr2 = " +
                "'" + shipToAddr.getStreetName2().trim() + "',";
        else
            queryStr += "shipaddr2 = ' ',";

        queryStr += "shipcity = " + "'" + shipToAddr.getCity().trim() + "',"
            + "shipstate = " + "'" + shipToAddr.getState().trim() + "',"
            + "shipzip = " + "'" + shipToAddr.getZipCode().trim() + "',"
            + "shipcountry = " + "'" + shipToAddr.getCountry().trim() + "',"
            + "billaddr1 = " + "'" + billToAddr.getStreetName1().trim() + "',";

        if (billToAddr.getStreetName2() != null)
            queryStr += "billaddr2 = " + "'"
                + billToAddr.getStreetName2().trim() + "',";
        else
            queryStr += "billaddr2 = ' ',";

        queryStr +=  "billcity = " + "'" + billToAddr.getCity().trim() + "',"
            + "billstate = " + "'" + billToAddr.getState().trim() + "',"
            + "billzip = " + "'" + billToAddr.getZipCode().trim() + "',"
            + "billcountry = " + "'" + billToAddr.getCountry().trim() + "',"
            + "courier = " + "'" + carrier.trim() + "',"
            + "totalprice = " + "CONVERT(DECIMAL(10,2),"
            + formatPrice(totalPrice) + "),"
            + "shiptofirstname = " + "'" + shipToFirstName.trim() + "',"
            + "shiptolastname = " + "'" + shipToLastName.trim() + "',"
            + "billtofirstname = " + "'" + billToFirstName.trim() + "',"
            + "billtolastname = " + "'" + billToLastName.trim() + "',"
            + "creditcard = " + "'" + chargeCard.getCardNo().trim() + "',"
            + "exprdate = " + "'" + chargeCard.getExpiryDateString().trim()
            + "'," + "cardtype = " + "'" + chargeCard.getCardType().trim() + "'"
            + " WHERE orderid = " + orderId;

        Debug.println("queryString is: "+ queryStr);
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);
            if ( resultCount != 1 )
                throw new OrderDAOException
                    ("ERROR updating order in ORDER_TABLE!! resultCount = " +
                         resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while updating " +
                    "order : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void updateLineItem() throws OrderDAOException {
        LineItem LI;
        int resultCount;

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            for (Iterator it = lineItems.iterator() ; it.hasNext() ;) {
                LI = (LineItem) it.next();
                if (LI == null)
                    throw new OrderDAOException("LineItem is null");
                String queryStr = "UPDATE "+ DatabaseNames.LINE_ITEM_TABLE
                    + " SET itemid = " + "'"+ LI.getItemNo().trim() + "',"
                    + "quantity = " + LI.getQty() + ","
                    + "unitprice = " + "CONVERT(DECIMAL(10,2),"
                    + LI.getUnitPrice() + ")"
                    + " WHERE orderid = " + orderId
                    + " AND linenum = " + LI.getLineNo();

                Debug.println("queryString is: "+ queryStr);
                resultCount = stmt.executeUpdate(queryStr);
                if ( resultCount != 1 )
                    throw new OrderDAOException
                        ("ERROR updating LINE_ITEM_TABLE  !! resultCount = "
                                + resultCount);
            }
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while updating " +
                    "line item : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void updateOrderStatus() throws OrderDAOException {

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr = "UPDATE " + DatabaseNames.ORDER_STATUS_TABLE
                + " SET status = " + "'"+ status.trim() + "'"
                + " WHERE orderid = " + orderId;
            Debug.println("queryString is: "+ queryStr);
            int resultCount = stmt.executeUpdate(queryStr);
            if ( resultCount != 1 )
                throw new OrderDAOException
                    ("ERROR updating ORDER_STATUS_TABLE !! resultCount = "
                            + resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while updating " +
                    "order status : \n" + se);
        } finally {
            closeConnection();
        }
    }
}
