/*
 * $Id: Calendar.java,v 1.3 2000/09/20 05:23:26 brydon Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.util;

import java.util.Date;
import com.sun.xml.tree.XmlDocument;
import com.sun.xml.tree.ElementNode;

/**
 * This class represents a calender
 */
public class Calendar extends Object implements java.io.Serializable{

    public static final int MONTH = java.util.Calendar.MONTH;
    public static final int DATE = java.util.Calendar.DATE;
    public static final int YEAR = java.util.Calendar.YEAR;

    private int month;
    private int day;
    private int year;

    private Calendar(int year, int month, int day){
        this.month = month;
        this.day= day;
        this.year = year;
    }

    public static Calendar getInstance(){
        java.util.Calendar c = java.util.Calendar.getInstance();
        int m = c.get(java.util.Calendar.MONTH);
        int d = c.get(java.util.Calendar.DATE);
        int y = c.get(java.util.Calendar.YEAR);
        return new Calendar(y,m,d);
    }

    public int getMonth(){
        return month;
    }

    public int getDay(){
        return day;
    }

    public int getYear(){
        return year;
    }

    public void set(int year, int month, int day){
        this.month = month;
        this.day = day;
        this.year = year;
    }

    public void set(int target, int value){
        switch (target){
            case java.util.Calendar.MONTH :
                this.month = value;
                break;
            case java.util.Calendar.YEAR :
                this.year = value;
                break;
            case java.util.Calendar.DATE :
                this.day = value;
                break;
        }
    }

    public Date getTime(){
        return new Date(year, month, day);
    }

    public int get(int target){
        switch (target){
            case java.util.Calendar.MONTH : return this.month;
            case java.util.Calendar.YEAR : return this.year;
            case java.util.Calendar.DATE : return this.day;
            default: return -1;
        }

    }

    public void setTime(java.util.Date date){
        java.util.Calendar c = java.util.Calendar.getInstance();
        c.setTime(date);
        this.day = c.get(java.util.Calendar.DATE);
        this.month = c.get(java.util.Calendar.MONTH);
        this.year = c.get(java.util.Calendar.YEAR);
    }

    public void clear(){
        this.day = -1;
        this.month = -1;
        this.year = -1;
    }

   /**
    * @return the date encoded in the format  mm/yyyy
    */
    public String getExpiryDateString(){
        return ((month > 10)? "0" : "") + month + "/" + ((year > 10)? "0" : "") + year;
    }

   /**
    * @return the date encoded in the format  mm/dd/yyyy
    */
    public String getFullDateString(){
        return ((month + 1 < 10)? "0" : "") + (month + 1) + "/" + ((day < 10)? "0" : "") + day + "/" + ((year < 10)? "0" : "") + year;
    }

    public String toString(){
        return "[Year=" + year + ", Month=" + month + ", Day=" + day + "]";
    }

    public String getCloudscapeDateString(){
        return year + "-" + ((month + 1 < 10)? "0" : "") + (month + 1) + "-" + ((day < 10)? "0" : "") + day;
    }

    public ElementNode toXml(XmlDocument doc, String id) {
        ElementNode root = (ElementNode) doc.createElement("Date");
        if (id != null)
            root.setAttribute("Id", id);

        ElementNode node = (ElementNode) doc.createElement("Month");
        node.appendChild(doc.createTextNode(String.valueOf(month)));
        root.appendChild(node);

        node = (ElementNode) doc.createElement("Day");
        node.appendChild(doc.createTextNode(String.valueOf(day)));
        root.appendChild(node);

        node = (ElementNode) doc.createElement("Year");
        node.appendChild(doc.createTextNode(String.valueOf(year)));
        root.appendChild(node);

        return root;
    }
}
