/*
 * $Id: AccountHandler.java,v 1.15 2000/10/06 19:45:27 gmurray Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.petstore.control.ejb;

import java.rmi.RemoteException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.util.JNDINames;
import com.sun.j2ee.blueprints.petstore.util.EJBUtil;

import com.sun.j2ee.blueprints.petstore.util.SecurityAdapter;

import com.sun.j2ee.blueprints.customer.account.model.AccountModel;
import com.sun.j2ee.blueprints.customer.customer.ejb.Customer;
import com.sun.j2ee.blueprints.customer.customer.ejb.CustomerHome;
import com.sun.j2ee.blueprints.petstore.personalize.profilemgr.ejb.ProfileMgr;
import com.sun.j2ee.blueprints.petstore.personalize.profilemgr.ejb.ProfileMgrHome;
import com.sun.j2ee.blueprints.petstore.control.DuplicateAccountException;
import com.sun.j2ee.blueprints.petstore.control.EStoreEventException;
import com.sun.j2ee.blueprints.petstore.control.event.AccountEvent;
import com.sun.j2ee.blueprints.petstore.control.event.EStoreEvent;


public class AccountHandler extends StateHandlerSupport {

  public void perform(EStoreEvent event) throws EStoreEventException {
        AccountEvent ae = (AccountEvent)event;

        switch (ae.getActionType()) {
        case AccountEvent.CREATE_ACCOUNT: {
            Debug.println("AccountHandler (ejb): CREATE_ACCOUNT event");
            try { // test
                SecurityAdapter securityAdapter = null;
                String securityAdapterClassName = null;
                try { // try2
                    InitialContext ic = new InitialContext();
                    securityAdapterClassName = (String)ic.lookup(JNDINames.SECURITY_ADAPTER_CLASSNAME);
                } catch (javax.naming.NamingException ex) {
                   Debug.println("AccountHandler caught: " + ex);
                    // ignore.. we are working around it below.
                }

                // use j2ee user manager if not specified
                if (securityAdapterClassName == null){
                    Debug.println("Security Adapter not specified in deployment descriptor: using default J2ee Security Adapter");
                    securityAdapterClassName = "com.sun.j2ee.blueprints.petstore.util.J2eeSecurityAdapter";
                }
                Debug.println("AccountHandler: Security Adapter class name = " + securityAdapterClassName);
                try{
                    securityAdapter = (SecurityAdapter)Class.forName(securityAdapterClassName).newInstance();
                } catch (java.lang.InstantiationException ie){
                    Debug.println("AccountHandler caught: " + ie);
                } catch (java.lang.IllegalAccessException il){
                    Debug.println("AccountHandler caught: " + il);
                } catch (java.lang.ClassNotFoundException e){
                    Debug.println("AccountHandler caught: " + e);
                }
               Debug.println("AccountHandler: creating user account for " + ae.getUserId());
               if (securityAdapter != null) securityAdapter.addUser("default", "gold",  ae.getUserId(), ae.getPassword());
               Debug.println("AccountHandler: created user account for " + ae.getUserId());

               CustomerHome home = EJBUtil.getCustomerHome();
               Customer cust = home.create();
               cust.createAccount(ae.getUserId(), ae.getStatus(), ae.getContactInformation());
               Debug.println("AccountHandler: creating user profile for " + ae.getUserId());
               ProfileMgrHome pHome = EJBUtil.getProfileMgrHome();
               pHome.create(ae.getUserId(), ae.getExplicitInformation());

            } catch (java.rmi.RemoteException re) {
                throw new EStoreEventException("Error creating account: " + re);
            } catch (DuplicateKeyException dke) {
                Debug.println("AccountHandler account exists for " + ae.getUserId());
                throw new DuplicateAccountException("Account Exists");
            } catch (CreateException ce) {
                Debug.println("AccountHandler create excpetion: user exists for " + ae.getUserId());
                throw new DuplicateAccountException("Account Exists");
            }  catch (javax.naming.NamingException ce) {
                Debug.println("AccountHandler naming exception: " + ce);
                throw new EStoreEventException("Error creating account: " + ce);
            }
        } break;
          case AccountEvent.UPDATE_ACCOUNT: {
          Debug.println("AccountHandler (ejb): UPDATE_ACCOUNT event");
            try {
               Debug.println("AccountHandler (ejb): updating user account for " + ae.getUserId());
               CustomerHome home = EJBUtil.getCustomerHome();
               Customer cust = home.create();
               cust.changeContactInformation(ae.getContactInformation());

               Debug.println("AccountHandler (ejb): updating user profile for " + ae.getUserId());
               ProfileMgrHome pHome = EJBUtil.getProfileMgrHome();
               ProfileMgr profileBeanRef = pHome.findByPrimaryKey(ae.getUserId());
               Debug.println("AccountHandler (ejb), perform, updateAccountEvent, eInfo=" + ae.getExplicitInformation().toString());
               profileBeanRef.updateExplicitInformation(ae.getExplicitInformation());

            } catch (java.rmi.RemoteException re) {
                throw new EStoreEventException("Error updating account: " + re);
            } catch (CreateException ce) {
                throw new EStoreEventException("Error creating account: " + ce);
            } catch (javax.ejb.FinderException fe) {
                throw new EStoreEventException("Error finding account: " + fe);
            } catch (javax.naming.NamingException ce) {
                Debug.println("AccountHandler naming exception: " + ce);
                throw new EStoreEventException("Error updating account: " + ce);
            }
        } break;
        default:
            Debug.print("Error: not implemented yet");
            break;
        }
  }
}

