/*
 * $Id: RequestProcessor.java,v 1.13 2000/10/06 22:50:53 gmurray Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */
package com.sun.j2ee.blueprints.petstore.control.web;

import java.util.Collection;
import java.util.HashMap;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Locale;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.util.JNDINames;
import com.sun.j2ee.blueprints.petstore.util.WebKeys;
import com.sun.j2ee.blueprints.petstore.util.SecurityAdapter;
import com.sun.j2ee.blueprints.petstore.control.event.EStoreEvent;
import com.sun.j2ee.blueprints.petstore.control.event.LogoutEvent;
import com.sun.j2ee.blueprints.petstore.control.event.LoginEvent;
import com.sun.j2ee.blueprints.petstore.control.event.AccountEvent;
import com.sun.j2ee.blueprints.petstore.control.EStoreEventException;
import com.sun.j2ee.blueprints.petstore.control.LoginFailedException;
import com.sun.j2ee.blueprints.petstore.control.web.ModelUpdateNotifier;
import com.sun.j2ee.blueprints.petstore.control.web.ScreenFlowManager;


/**
 * This is the web tier controller for the sample application.
 *
 * This class is responsible for processing all requests received from
 * the Main.jsp and generating necessary events to modify data which
 * are sent to the ShoppingClientControllerWebImpl.
 *
 */
public class RequestProcessor {


    //private ModelManager mm;
    private ScreenFlowManager screenManager;
    private ServletContext context;
    //private ModelUpdateNotifier mun;
    private RequestToEventTranslator eventTranslator;
    private SecurityAdapter securityAdapter;

    /** Empty constructor for use by the JSP engine. */
    public RequestProcessor() {}


    public void init(ServletContext context) {
        this.context = context;
        screenManager = (ScreenFlowManager)context.getAttribute(WebKeys.ScreenManagerKey);

        eventTranslator = new RequestToEventTranslator(this, screenManager);
        String securityAdapterClassName = null;
        String serverType = null;
        try {
            InitialContext ic = new InitialContext();
            securityAdapterClassName = (String)ic.lookup(JNDINames.SECURITY_ADAPTER_CLASSNAME);
            serverType = (String)ic.lookup(JNDINames.SERVER_TYPE);
            context.setAttribute("serverType", serverType);
        } catch (NamingException ex){
            Debug.println("Security Adapter not specified in deployment descriptor: using default J2ee Security Adapter");
        }

        // use j2ee user manager if not specified
        if (securityAdapterClassName == null){
            securityAdapterClassName = "com.sun.j2ee.blueprints.petstore.util.J2eeSecurityAdapter";
        }
        Debug.println("RequestProcessor: Security Adapter class name = " + securityAdapterClassName);
        try{
            securityAdapter = (SecurityAdapter)Class.forName(securityAdapterClassName).newInstance();
        } catch (java.lang.InstantiationException ie){
            Debug.println("RequestProcessor caught: " + ie);
        } catch (java.lang.IllegalAccessException il){
            Debug.println("RequestProcessor caught: " + il);
        } catch (java.lang.ClassNotFoundException e){
            Debug.println("RequestProcessor caught: " + e);
        }
    }

    /**
    * This method is the core of the RequestProcessor. It receives all requests
    *  and generates the necessary events.
    */
    public void processRequest(HttpServletRequest request)
        throws EStoreEventException {
        EStoreEvent event = null;
        ModelManager mm = (ModelManager)request.getSession().getAttribute(WebKeys.ModelManagerKey);
        ShoppingClientControllerWebImpl scc = (ShoppingClientControllerWebImpl)request.getSession().getAttribute(WebKeys.WebControllerKey);
        if (scc == null) {
            scc = new ShoppingClientControllerWebImpl(request.getSession());
            mm.setSCC(scc);
            request.getSession().setAttribute(WebKeys.WebControllerKey, scc);
        }
        try {
            checkForWebServerLogin(request, mm, scc);
            event = eventTranslator.processRequest(request);
            if (event != null) {
                Collection updatedModelList = scc.handleEvent(event);
                /* This portion of code automatically logs a user into the
                 * web server when a new account is created. The user will
                 * not see the form based login page.
                 */
                if ((event instanceof AccountEvent) && ((AccountEvent)event).getActionType() == AccountEvent.CREATE_ACCOUNT) {
                    if (securityAdapter != null) {
                        securityAdapter.loginUser(((AccountEvent)event).getUserId(), ((AccountEvent)event).getPassword(), request.getSession());
                    }
                    mm.notifyListeners(updatedModelList);
                    checkForWebServerLogin(request, mm, scc);
                    // invalidate current session and replace it with a new session
                }
                mm.notifyListeners(updatedModelList);
                if (event instanceof LogoutEvent) {
                    request.getSession().invalidate();
                    screenManager = null;
                    //remove the EJB
                    scc.remove();
                    // get new session and put in a new gui controller
                    HttpSession validSession = request.getSession(true);
                    mm = new ModelManager();
                    validSession.setAttribute(WebKeys.ModelManagerKey, mm);
                    mm.init(context, validSession);
                    //this.init(context, validSession);
                    //validSession.setAttribute(WebKeys.RequestProcessorKey, this);
                }
            }
        } catch (LoginFailedException lfe) {}
    }

    private void checkForWebServerLogin(HttpServletRequest request, ModelManager mm, ShoppingClientControllerWebImpl scc){
        // check if the user has logged in using the form based login
        // spec says that userprincipal to null
        if ((request.getUserPrincipal() != null) &&
            !request.getUserPrincipal().getName().equals("guest") &&
            !mm.getAccountModel().isLoggedIn()) {

            EStoreEvent loginEvent = new LoginEvent();
            try {
                Collection updatedModelList = scc.handleEvent(new LoginEvent());
                mm.notifyListeners(updatedModelList);
                // get the language here
                String language = mm.getProfileMgrModel().getExplicitInformation().getLangPref();
                Debug.println("RequestProcessor: settting language to : " + language);
                // set the locale here
                Locale locale = Locale.US;
                if (language.equals("English")) locale = Locale.US;
                else if (language.equals("Japanese")) locale = Locale.JAPAN;
                request.getSession().setAttribute(WebKeys.LanguageKey, locale);
                Debug.println("Account Handler set language to: " + language);
            } catch (LoginFailedException lfe) {
            } catch (EStoreEventException se) {
                Debug.print(se);
            }
        }
    }
}
