/*
 * $Id: ProfileMgrDAO.java,v 1.15 2000/10/08 22:27:03 inder Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.petstore.personalize.profilemgr.ejb;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.sql.DataSource;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;

import com.sun.j2ee.blueprints.petstore.personalize.util.JNDINames;
import com.sun.j2ee.blueprints.petstore.personalize.util.DatabaseNames;
import com.sun.j2ee.blueprints.petstore.personalize.util.ExplicitInformation;
import com.sun.j2ee.blueprints.util.tracer.Debug;

/**
 * This class encapsulates all the JDBC calls made by the ProfileMgrEJB.
 * Actual logic of inserting/fetching/updating/deleting  the data in
 * relational database tables to mirror the state of ProfileMgrEJB is
 * implemented here.
 */
public class ProfileMgrDAO {

    private String userId;
    private ExplicitInformation eInfo;

    private Connection dbConnection = null;
    private DataSource datasource   = null;

    public ProfileMgrDAO() throws ProfileMgrDAOException {
        try {
            InitialContext ic = new InitialContext();
            datasource = (DataSource) ic.lookup(JNDINames.ESTORE_DATASOURCE);
        } catch (NamingException ne) {
            throw new ProfileMgrDAOException("NamingException while looking" +
                                                " up DataSource Connection " +
                                                 JNDINames.ESTORE_DATASOURCE +
                                                       ": \n" + ne.getMessage());
        }
    }

    // get and set methods for the instance variables

    public String getUserId(){
        return userId;
    }

    public ExplicitInformation getExplicitInformation() {
        return eInfo;
    }

    public void setUserId( String userId) {
        this.userId = userId;
    }

    public void setExplicitInformation(ExplicitInformation eInfo) {
        this.eInfo = eInfo;
    }

    public void create() throws ProfileMgrDAOException {
        insertProfile();
    }

    public void load() throws ProfileMgrDAOException {
        selectProfile();
    }

    public void store() throws ProfileMgrDAOException {
        updateProfile();
    }

    public void remove() throws ProfileMgrDAOException {
        deleteProfile();
    }

    public String findByPrimaryKey() throws ProfileMgrDAOException {
        if (userExists(userId))
            return (userId);
        throw new ProfileMgrDAOException (
                        "ProfileMgrDAO: primary key not found:" + userId);
    }

    public String getBanner(String favCategory) throws
                                        ProfileMgrDAOException {
        String favCat = favCategory;
        String bName = null;

        String qstr = "SELECT favcategory, bannername FROM " +
            DatabaseNames.BANNERDATA_TABLE +
            " WHERE favcategory =  " + "'" + favCat.trim() + "'";
        Debug.println("Query String is:" + qstr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            while (rs.next()) {
                int i = 1;
                favCat = rs.getString(i++).trim();
                bName = rs.getString(i++).trim();
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while getting " +
                        "Banner for favorite category " + favCategory + " : " + se);
        } finally {
            closeConnection();
        }
        return bName;
    }

    private boolean userExists(String userId) throws ProfileMgrDAOException {

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr ="SELECT userid FROM " +
                            DatabaseNames.PROFILE_TABLE
                            + " WHERE userid = " + "'" + userId.trim() + "'";

            ResultSet result = stmt.executeQuery(queryStr);
            if(!result.next()) {
                stmt.close();
                return false;
            } else {
                userId = result.getString(1);
                stmt.close();
                return true;
            }
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while checking " +
                            "for presence of user id " + userId + " : " + se);
        } finally {
            closeConnection();
        }
    }

    private boolean isValidData () {
        if ( (userId == null) ||
             (eInfo.getLangPref() == null) ||
             (eInfo.getFavCategory() == null))
            return (false);
        else
            return (true);
    }

    private void insertProfile() throws ProfileMgrDAOException {

        if(!isValidData())
            throw new ProfileMgrDAOException(
                            "ProfileMgrDAO: Illegal data values for insert");

        if (userExists(userId))
            throw new ProfileMgrDAOException("ProfileMgrDAO: Profile Exists: "
                                        + userId );

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();

            String queryStr = "INSERT INTO " + DatabaseNames.PROFILE_TABLE +
                    "(userid,langpref,mylistopt,banneropt,"
                    + "favcategory)" + "VALUES ("
                    + "'" + userId.trim() + "',"
                    + "'" + eInfo.getLangPref() + "',"
                    + (eInfo.getMyListOpt()? 1:0) + ","
                    + (eInfo.getBannerOpt()? 1:0) + ",'"
                    + eInfo.getFavCategory() + "' )";

            int resultCount = stmt.executeUpdate(queryStr);

            if ( resultCount != 1 )
                throw new ProfileMgrDAOException(
                    "ProfileMgrDAO: ERROR in PROFILE_TABLE INSERT !! " +
                    "resultCount = " + resultCount);
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while inserting " +
                            "profile : " + se);
        } finally {
            closeConnection();
        }
    }

    private void selectProfile() throws ProfileMgrDAOException {

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr = "SELECT "+
                    "userid,langpref,mylistopt,banneropt,favcategory"+
                    " FROM " + DatabaseNames.PROFILE_TABLE +
                    " WHERE userid = " + "'" + userId.trim() + "'";

            ResultSet result = stmt.executeQuery(queryStr);

            if ( !result.next() )
                throw new ProfileMgrDAOException(
                        "ProfileMgrDAO: No record for primary key " + userId);

            int i = 1;
            userId = result.getString(i++);
            String langPref = result.getString(i++);
            int myListOpt = result.getInt(i++);
            int bannerOpt = result.getInt(i++);
            String favCategory = result.getString(i++);
            eInfo = new ExplicitInformation(langPref,favCategory,
                                    (myListOpt==1),(bannerOpt==1));
            stmt.close();
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while selecting " +
                            "profile of user id " + userId + " : " + se);
        } finally {
            closeConnection();
        }
    }

    private void deleteProfile() throws ProfileMgrDAOException {

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr = "DELETE FROM " + DatabaseNames.PROFILE_TABLE
                    + " WHERE userid = " + "'" + userId.trim() + "'";
            int resultCount = stmt.executeUpdate(queryStr);

            if (resultCount != 1)
                throw new ProfileMgrDAOException
                    ("ProfileMgrDAO: ERROR deleteing profile from " +
                        "PROFILE_TABLE!! resultCount = " + resultCount);
            stmt.close();
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while deleting " +
                            "user id " + userId + " : " + se);
        } finally {
            closeConnection();
        }
    }

    private void updateProfile() throws ProfileMgrDAOException {
        if (!isValidData())
            throw new ProfileMgrDAOException(
                "ProfileMgrDAO: Illegal profile data values for update");

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            String queryStr = "UPDATE " + DatabaseNames.PROFILE_TABLE + " SET "
                + "langpref = " + "'" + eInfo.getLangPref().trim() + "',"
                + "favcategory = " + "'" + eInfo.getFavCategory().trim() + "',"
                + "mylistopt = " + ((eInfo.getMyListOpt())? 1 :0)
                + ",banneropt = " + ((eInfo.getBannerOpt())? 1 :0)
                + " WHERE userid = " + "'" + userId.trim() + "'";
            Debug.println("Executing query: " + queryStr);
            int resultCount = stmt.executeUpdate(queryStr);
            if (resultCount != 1)
                throw new ProfileMgrDAOException
                    ("ProfileMgrDAO: ERROR updating profile in " +
                        "PROFILE_TABLE!! resultCount = " + resultCount);
            stmt.close();
        } catch(SQLException se) {
            Debug.print(se);
            throw new ProfileMgrDAOException("SQLException while updating " +
                            "profile of user id " + userId + " : " + se);
        } finally {
            closeConnection();
        }
    }

    private void getDBConnection() throws ProfileMgrDAOException {
        try {
            dbConnection = datasource.getConnection();
        } catch (SQLException se) {
            throw new ProfileMgrDAOException("SQLException while getting " +
                                          "DB connection " + se.getMessage());
        }
    }

    private void closeConnection() throws ProfileMgrDAOException {
        try {
            if (dbConnection != null && !dbConnection.isClosed()) {
                dbConnection.close();
            }
        } catch(SQLException se) {
            throw new ProfileMgrDAOException("SQLException while closing " +
                                          "DB connection " + se.getMessage());
        }
    }
}



